#include "StdAfx.h"
#include ".\componentscriptletplugin.h"
#include "XMLUtil.h"
#include "ScriptletConstant.h"
#include "Application.h"
#include "ScriptletContainerFactory.h"
#include "PropertyEvent.h"
#include "Binding.h"
#include "DataContext.h"

CComponentScriptletPlugin::CComponentScriptletPlugin(void)
{
}

CComponentScriptletPlugin::~CComponentScriptletPlugin(void)
{
}
BOOL CComponentScriptletPlugin::PreModifyComponent(CContainer* pParent, CComponent* pTarget, TiXmlElement* pNode)
{
	return TRUE;
}
BOOL CComponentScriptletPlugin::ModifyComponent(CContainer* pParent, CComponent* pTarget, TiXmlElement* pNode)
{
	BOOL bSuccess = FALSE;
	ATLASSERT(pTarget && pNode);
	if (pTarget == NULL || pNode == NULL) return bSuccess;

	TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};
	long lValue = 0;

	USES_CONVERSION;

	//inherit
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_INHERIT_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_INHERIT_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		TiXmlElement* pInheritNode = Application::LoadLayoutNode(szValue);
		if ( NULL != pInheritNode )
		{
			ModifyComponent(pParent, pTarget, pInheritNode);
		}
	}
	// id
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_ID_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		pTarget->SetName(szValue);
	}
	CRect rcBound;
	pTarget->GetOriginalBounds(&rcBound);
	//rcBound.SetRect(LONG_MAX, LONG_MAX, LONG_MAX, LONG_MAX);
	// x
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_X_NODE), lValue))
	{
		rcBound.left = lValue;
	}
	// y
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_Y_NODE), lValue))
	{
		rcBound.top = lValue;
	}
	// width
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_WIDTH_NODE), lValue))
	{
		rcBound.right = CComponent::GetRealValue(rcBound.left) + lValue;
	}
	// height
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_HEIGHT_NODE), lValue))
	{
		rcBound.bottom = CComponent::GetRealValue(rcBound.top) + lValue;
	}
	pTarget->MoveWindow(&rcBound);

	// enabled
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_ENABLED_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		BOOL bEnabled = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
		pTarget->EnableWindow(bEnabled);
	}	
	// eventroute
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_EVENTROUTE_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		BOOL bEnableEventRoute = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
		pTarget->SetEventRoute(bEnableEventRoute);
	}
	// onproperty
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_ONPROPERTY_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_ONPROPERTY_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		IPropertyChangedListener* propertylistener = CScriptletContainerFactory::get_instance()->CreatePropertyChangeListener(szValue);
		if ( NULL != propertylistener )
		{
			pTarget->AddPropertyChangedListener(propertylistener);
			propertylistener->AutoRelease();
		}
	}
	
	bSuccess = TRUE;
	return bSuccess;
}
BOOL CComponentScriptletPlugin::PostModifyComponent(CContainer* pParent, CComponent* pTarget, TiXmlElement* pNode)
{
	TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};
	long lValue = 0;
	USES_CONVERSION;
	// visible
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_VISIBLE_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		BOOL bVisible = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
		pTarget->SetVisible(bVisible);
	}
	return TRUE;
}
CComponent* CComponentScriptletPlugin::CreateComponent(CContainer* pParent, TiXmlElement* pNode)
{
	if (pParent == NULL) return NULL;

	CComponent* pComponent = NULL;

	pComponent = CreateRefineComponent(pParent, pNode);
	if (pComponent != NULL)
	{
		return pComponent;
	}

	pComponent = CreateComponentByName(pNode);
	if (pComponent == NULL)
	{
		pComponent = _CreateComponent(pNode);
	}
	
	if (NULL == pComponent) 
	{
		ATLTRACE(_T("Failed _CreateComponent \n"));
		return NULL;
	}	
	pComponent->SetParent(pParent);
	
	return pComponent;
}
BOOL CComponentScriptletPlugin::GenerateComponent(CContainer* pParent, CComponent* pComponent, TiXmlElement* pNode)
{
	PreModifyComponent(pParent, pComponent, pNode);
	if (ModifyComponent(pParent, pComponent, pNode))
	{
		PostModifyComponent(pParent, pComponent, pNode);
		pComponent->ModifyComponent(pParent, pNode);

		pComponent->Create();
		pParent->Add(pComponent);
		CheckBinding(pParent, pComponent, pNode);
		return TRUE;
	}
	delete pComponent;
	return FALSE;
}
CComponent* CComponentScriptletPlugin::CreateRefineComponent(CContainer* pParent, TiXmlElement* pNode)
{
	BOOL bRedefine = FALSE;
	TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};
	// redefine
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_REDEFINE_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
		XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_REDEFINE_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		bRedefine = _tcsicmp(szValue, SCRIPTLET_TRUE_VALUE) == 0;
	}
	if (bRedefine == FALSE)
	{
		return NULL;		
	}
	// id
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_ID_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		CComponent* pComponent = pParent->FindName(szValue);
		if (pComponent != NULL)
		{
			return pComponent;
		}
	}
	return NULL;
}
CComponent* CComponentScriptletPlugin::CreateComponentByName(TiXmlElement* pNode)
{
	CContainer* pContainer = NULL;
	TCHAR szValue[SCRIPTLET_MAX_VALUE] = {0};

	//class
	if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_CLASS_NODE), szValue, SCRIPTLET_MAX_VALUE))
	{
		pContainer = CScriptletContainerFactory::get_instance()->CreateScriptContainer(szValue);
	}
	if ( NULL == pContainer )
	{
		//inherit
		if (XMLUtil::GetAttrValue(pNode, CT2CA(SCRIPTLET_INHERIT_NODE), szValue, SCRIPTLET_MAX_VALUE) ||
			XMLUtil::GetValueAsAttr(pNode, CT2CA(SCRIPTLET_INHERIT_NODE), szValue, SCRIPTLET_MAX_VALUE))
		{
			TiXmlElement* pInheritNode = Application::LoadLayoutNode(szValue);
			if ( NULL != pInheritNode )
			{
				if (XMLUtil::GetAttrValue(pInheritNode, CT2CA(SCRIPTLET_CLASS_NODE), szValue, SCRIPTLET_MAX_VALUE))
				{
					pContainer = CScriptletContainerFactory::get_instance()->CreateScriptContainer(szValue);
				}
			}
		}		
	}
	return pContainer;
}
void CComponentScriptletPlugin::CheckBinding(CContainer* pParent, CComponent* pTarget, TiXmlElement* pNode)
{
	ATLASSERT(pNode != NULL);

	USES_CONVERSION;
	TiXmlAttribute* attr = pNode->FirstAttribute();
	while (attr != NULL)
	{
		CString attrName = CA2CT(attr->Name());
		CString attrValue = CA2CT(attr->Value());
		if (_T('{') == attrValue.GetAt(0) && _T('}') == attrValue.GetAt(attrValue.GetLength()-1))
		{
			attrValue = attrValue.Mid(1, attrValue.GetLength()-2);

			if (_tcsncmp(_T("Binding"), attrValue, 7) == 0)
			{
				if (ParseBinding(pParent, pTarget, attrName, attrValue))
				{
				}				
			}			
		}	
		attr = attr->Next();
	}
}
BOOL CComponentScriptletPlugin::ParseBinding( CContainer* pParent, CComponent* pTarget, CString attrName, CString &attrValue )
{
	int nToken = 8;//8 = _tcslen(_T("Binding")) + 1
	CString bindingInfo = attrValue.Tokenize(_T(" "), nToken);

	IDataContext* dataContext = NULL;
	CContainer* bindingContainer = NULL;
	CString propertyPath;
	IValueConverter* valueConverter = NULL;

	while (nToken != -1 && bindingInfo.GetLength() > 0)
	{
		int nToken2 = 0;
		CString bindingProperty = bindingInfo.Tokenize(_T(","), nToken2);

		while (nToken2 != -1 && bindingProperty.GetLength() > 0)
		{
			int nToken3 = 0;
			CString propertyKey = bindingProperty.Tokenize(_T("="), nToken3);
			CString propertyValue = bindingProperty.Tokenize(_T("="), nToken3);

			if (_tcsicmp(propertyKey, _T("Source")) == 0)
			{
				dataContext = IDataContext::FindDataContext(propertyValue);
			}
			else if (_tcsicmp(propertyKey, _T("Path")) == 0)
			{
				propertyPath = propertyValue;
			}
			else if (_tcsicmp(propertyKey, _T("Converter")) == 0)
			{
				valueConverter = CScriptletContainerFactory::get_instance()->CreateValueConverter(propertyValue);
			}
			bindingProperty = bindingInfo.Tokenize(_T(","), nToken2);
		}
		bindingInfo = attrValue.Tokenize(_T(" "), nToken);
	}
	if (dataContext == NULL)
	{
		CContainer* findContainer= NULL;
		CContainer* startContainer = dynamic_cast<CContainer*>(pTarget);
		if (startContainer == NULL)
		{
			startContainer = pParent;
		}
		dataContext = startContainer->FindDataContext(&findContainer);
		if (findContainer != NULL)
		{
			bindingContainer = findContainer;
		}
	}
	if (dataContext != NULL && propertyPath.GetLength() > 0)
	{
		Binding* binding = NULL;
		if (bindingContainer != NULL)
		{
			binding = bindingContainer->CreateBinding();
		}
		else
		{
			binding = pTarget->CreateBinding();
		}
		binding->SetSource(dataContext, propertyPath);
		binding->SetValueConverter(valueConverter);
		pTarget->SetBinding(attrName, binding);
		
		if (valueConverter != NULL)
		{
			valueConverter->AutoRelease();
		}
		return TRUE;
	}
	return FALSE;
}